import itertools
import textwrap
import shutil
import functools
from ..lib import Program
from ..signature import Signature, serialize_endpoint
from .doc import Doc

def _section_prefix(prog, x):
    for section in sorted(prog, key=len, reverse=True):
        if tuple(section[:len(x)]) == x:
            return x
    return tuple()

@serialize_endpoint
def _endpoints(section, s):
    signature = ''
    for x in s.positional:
        signature += ' %s' % x.name
    for x in s.keyword1:
        signature += ' [%s]' % x.name
    if s.var_positional:
        signature += ' [%s ...]' % s.var_positional.name
    return section, signature[1:]

def _join(f):
    def decorator(*args, **kwargs):
        return '\n'.join(f(*args, **kwargs))
    return decorator

@_join
def _format_arg(prefix, indent, x):
    columns, _ = shutil.get_terminal_size((80, 20))
    argname, desc = x
    whitespace = ' ' * len(argname)
    n = columns - len(argname) - indent - len(prefix) - 2
    
    first = True
    for right in textwrap.wrap(desc, n):
        if first:
            left = argname + ': '
            first = False
        else:
            left = whitespace + '  '
        yield (' ' * indent) + left + right

def usage(name, f, h):
    prog = Program(f)
    section = _section_prefix(prog, h.section)
    p = {
        'name': name,
        'message': h.message,
        'endpoints': _endpoints(prog, section),
    }
    if p['message']:
        yield 'error: %(message)s' % p
    for i, (section, signature) in enumerate(p['endpoints']):
        q = {
            'prefix': 'usage: ' if i == 0 else '       ',
            'name': p['name'],
            'sub': (' ' + ' '.join(section)).rstrip() + ' ',
            'signature': signature,
            'sep': ' [--] ' if any(signature) else ' '
        }
        yield '%(prefix)s%(name)s%(sub)s[-help] [options]%(sep)s%(signature)s' % q

def man(name, f, h):
    columns, _ = shutil.get_terminal_size((80, 20))

    prog = Program(f)
    section = _section_prefix(prog, h.section)
    g = prog.get(section)
    fdoc = Doc(g)
    p =  {
        'name': name,
        'endpoints': _endpoints(prog, section),
        'description': fdoc.desc,
        'args': (_format_arg(' ', 2, a) for a in fdoc.args),
        'kwargs': (_format_arg('-', 2, a) for a in fdoc.kwargs),
    }

    yield 'SYNOPSIS'
    for section, signature in p['endpoints']:
        q = {
            'pointer': '  ' if section[len(h.section):] else '> ',
            'name': p['name'],
            'sub': (' ' + ' '.join(section)).rstrip() + ' ',
            'signature': signature,
            'sep': ' [--] ' if any(signature) else ' '
        }
        yield '%(pointer)s%(name)s%(sub)s[-help] [options]%(sep)s%(signature)s' % q
    if p['description']:
        yield 'DESCRIPTION'
        for line in textwrap.wrap(p['description'], columns-2):
            yield '  ' + line
    if p['args']:
        yield 'INPUTS'
        for arg in p['args']:
            yield arg
        try:
            arg
        except NameError:
            yield '  (None)'
    yield 'OPTIONS'
    for kwarg in p['kwargs']:
        yield kwarg
    yield 'DETAIL'
    yield '  Run "-help" with a particular subcommand for more help.'



from funmap import funmap
from itertools import accumulate
from ..exceptions import HoretuCouldNotParse
from ..lib import Program

Input = funmap('func', 'section', 'positionals', 'flags')

@funmap('func', 'section')
def getsection(is_cli, function, argv):
    prog = Program(function)
    for section in sorted(prog, key=len, reverse=True):
        if tuple(argv[:len(section)]) == section:
            valid = True
            break
    else:
        valid = False
    
    if valid and section in prog:
        f = prog[section]
    else:
        if is_cli:
            def allow(a):
                return not a.startswith('-')
        else:
            def allow(a):
                return True
        for section in accumulate((a,) for a in argv if allow(a)):
            if section not in prog:
                break
        else:
            section = tuple()
        def f():
            if argv:
                msg = 'Bad subcommand'
            else:
                msg = 'Needs subcommand'
            raise HoretuCouldNotParse(section=section, message=msg)
    return {'func': f, 'section': section}



import logging
import os
import re
from functools import wraps
from ..lib import Program
from ..signature import Signature, serialize_endpoint
from .doc import Doc

logger = logging.getLogger(__name__)

@serialize_endpoint
def _endpoints(section, s):
    signature = ''
    for x in section:
        signature += '/%s' % x
    for x in s.positional:
        signature += '/:%s' % x.name
    for x in s.keyword1:
        signature += '/[%s]' % x.name
    if s.var_positional:
        signature += '/&lt;%s&gt;' % s.var_positional.name
    signature += '?(option=...)'
    return signature

def usage(f, h):
    prog = Program(f)
    if hasattr(prog.get(h.section), '__call__'):
        sections = []
        endpoints = _endpoints(prog, h.section)
    else:
        sections = ['/'.join(h.section + s) for s in prog]
        endpoints = []

    message = h.message
    sections = sections

    if message:
        yield '<em>%s</em>' % message
    yield '<ul>'
    for signature in sections:
        msg = '<li><a href="%(signature)s">%(signature)s</a></li>'
        yield msg % {'signature': signature}
    for signature in endpoints:
        yield '<li><code>%s</code></li>' % signature
    yield '</ul>'

def man(f, h):
    prog = Program(f)
    g = prog.get(h.section)
    fdoc = Doc(g)

    print(prog)
    is_section = hasattr(g, 'items')
    is_callable = hasattr(g, '__call__')
    endpoints = _endpoints(prog, h.section)
    description = list(filter(None, re.split('r[\n\r]{2,}', fdoc.desc)))
    args = list(fdoc.args)
    kwargs = fdoc.kwargs

    yield '''\
<h2>Synopsis<h2>
<ul>
'''
    for i, signature in enumerate(endpoints):
        if is_section and i == 0:
            yield '<strong>'
        yield signature
        if is_section and i == 0:
            yield '</strong>'
    yield '</ul>'
    if description:
        yield '<h2>Description</h2>'
        for p in description:
            yield '<p>%s</p>' % p
    if is_callable:
        if args:
            yield '''\
<h3>Inputs</h3>
<dl>
'''
        for pair in args:
            yield '<dt>%s</dt><dd>%s</dt>' % pair
        if len(args) == 0:
            yield '<p>(None)</p>'
        yield '''\
</dl>
<h3>Options</h3>
<dl>
'''
        for pair in kwargs:
            yield '<dt>%s</dt><dd>%s</dd>' % pair
        yield '</dl>'
    yield '''\
<h2>Detail</h2>
<p>
  Put "help" in the query string ("?help") for more help about a
  particular endpoint.
</p>
'''



import os
from abc import ABCMeta
from functools import wraps, partial

def AppIter(x):
    '''
    :param x: Anything (returned from the underlying program)
    :rtype: Iterable of bytes
    :returns: Output as bytes
    '''
    if x is None:
        raise StopIteration
    elif isinstance(x, bytes):
        yield x
    elif isinstance(x, str):
        yield x.encode('utf-8')
    elif hasattr(x, '__iter__'):
        for y in x:
            yield from AppIter(y)
    else:
        yield format(x).encode('utf-8')



from . import lib

@lib.Input
def wsgi(f, environ):
    from webob import Request
    return _wsgi(f, Request(environ))

def _wsgi(f, req):
    pos = req.path_info.split('/')[1:]
    x = lib.getsection(False, f, pos)

    def _flatten(multidict):
        for k in multidict:
            for v in multidict.getall(k):
                yield k, v
    return {
        'func': x.func,
        'section': x.section,
        'positionals': pos[len(x.section):],
        'flags': list(_flatten(req.GET)),
    }



import pytest
from . import triedict, AmbiguousKey, KeyNotFound

@pytest.fixture
def w():
    return triedict(key='value')

@pytest.fixture
def x():
    return triedict(chain=1, chainsaw=2, calque=3, loanword=4)

@pytest.fixture
def y():
    return triedict({
        (1, 2): 'a',
        (1, 2, 3): 'b',
        (10, 11, 12, 13): 'c',
    })

@pytest.fixture
def z():
    return triedict(a_b=1, a_bcdef=2, a_bbb=3, aaaaaaaaa=4, aaabb=5)

@pytest.mark.parametrize('key, value', [
    ('a', AmbiguousKey),
    ('a_b', 1),
    ('a_bc', 2),
    ('aaxbb', KeyNotFound),
    ('aaaaaaa', 4),
    ('aaaaaaaaaaaaaaa', KeyNotFound),
])
def test_errors(z, key, value):
    if isinstance(value, type) and issubclass(value, Exception):
        with pytest.raises(value):
            z[key]
    else:
        assert z[key] == value

_argvalues = [
    ('aoeu', True),
    (b'aoeu', False),
    (1234, False),
]
@pytest.mark.parametrize('key, ok', _argvalues)
def test_check_type(x, key, ok):
    if ok:
        x._check_type(key)
    else:
        with pytest.raises(TypeError):
            x._check_type(key)

def test_repr(x):
    assert 'triedict(' in repr(x)
    assert 'chainsaw\': 2' in repr(x)

def test_getitem(x):
    assert x['chain'] == 1
    with pytest.raises(AmbiguousKey):
        x['chai']
    assert x['calque'] == 3
    assert x['l'] == 4

def test_setitem(x):
    x['aoeu'] = 9
    x._trie['a']['o']['e']['u'][None] == 9

def test_delitem(x):
    with pytest.raises(NotImplementedError):
        del(x['printer'])

def test_iter(x):
    assert list(sorted(x)) == ['calque', 'chain', 'chainsaw', 'loanword']

def test_len(x):
    assert len(x) == 4

_cases = [
    ('key', 'value'),
    ('u', KeyNotFound),
    ('', 'value'),
]
@pytest.mark.parametrize('k, v', _cases)
def test_getitem_one(w, k, v):
    if isinstance(v, type) and issubclass(v, Exception):
        with pytest.raises(v):
            w[k]
    else:
        assert w[k] == v

def test_getitem_tuple(y):
    assert y[(10,)] == 'c'

def test_iter_tuple(y):
    assert list(sorted(y)) == [
        (1, 2),
        (1, 2, 3),
        (10, 11, 12, 13),
    ]



import os
from functools import partial

# These are so short that they're really just examples.
InputFile = partial(open, mode='r')
OutputFile = partial(open, mode='w')

def InputDirectory(x):
    if os.path.isdir(x):
        return x
    else:
        raise ValueError('No such directory: ' + str(x))

def OutputDirectory(x):
    os.makedirs(x, exist_ok=True)
    return x



import os
import inspect
from enum import Enum
from funmap import funmap
from functools import partial
from . import config
from .lib import Program
from .signature import Signature
from .templates import AppIter
from .exceptions import (
    HoretuException, Error,
    HoretuShowHelp, HoretuCouldNotParse,
)

ANNOTATE_POS = 'Cannot annotate positional argument "%s"'

def Output(config_file, config_sep, function, inputs):
    '''
    :type config_file: str or None
    :param config_file: Location of the configuration file, or None
    :param str config_sep: Separator character between command and
        subcommand names in the configuration file (Default is " ".)
    :type function: dict, list, or callable
    :param function: The function or collection of functions
    :param Inputs inputs: Specification of the section,
        positional arguments, and keyword arguments
    :returns: Either the payload (iter of bytes) or a :py:class:`HoretuException`
    '''
    if config_file:
        if not os.path.exists(config_file):
            with open(config_file, 'w') as config_fp:
                config.write(Program(function), config_fp, config_sep)
        with open(config_file, 'r') as config_fp:
            defaults = config.read(config_fp, config_sep)
    else:
        defaults = {}

    s = Signature(inputs.func)
    g = _with_defaults(inputs.func, defaults, s)

    fail = None
    args = reify_args(s, inputs.positionals)
    if isinstance(args, HoretuException):
        fail = args

    kwargs = reify_kwargs(s, inputs.flags)
    if isinstance(kwargs, HoretuShowHelp):
        fail = kwargs

    if fail == None:
        y = g(*args, **kwargs)
        return AppIter(y)
    else:
        fail.section = inputs.section
        raise fail

def reify_args(s, str_args):
    '''
    :param Signature s:
    '''
    xs = iter(str_args)
    args = []
    fail = None

    vp = [s.var_positional] if s.var_positional else []
    for a in s.positional + s.keyword1 + vp:
        if not a.supports_positional:
            return TypeError(ANNOTATE_POS % a.name)

    for a in s.positional:
        try:
            x = next(xs)
        except StopIteration:
            fail = HoretuCouldNotParse(message='Not enough arguments')
        else:
            args.append(a.call(x))

    for a in s.keyword1:
        try:
            x = next(xs)
        except StopIteration:
            break
        else:
            args.append(a.call(x))

    a = s.var_positional
    if a:
        args.extend(map(a.call, xs))
    else:
        xs = list(xs)
        if xs:
            fail = HoretuCouldNotParse('Extra arguments -- %s' % ' '.join(xs))

    if fail == None:
        return args
    else:
        return fail

def reify_kwargs(s, str_kwargs):
    kwargs = {}
    for name, value in str_kwargs:
        a = s.keyword2.get(name)
        if a:
            try:
                kwargs[name] = a.call(value)
            except ValueError as e:
                return HoretuCouldNotParse(e.args[0])
            except HoretuException as e:
                return e
    return kwargs

def _with_defaults(f, defaults, sig):
    def w(*args, **kwargs):
        for key, value in defaults.items():
            if key in sig.keyword2 and key not in kwargs:
                kwargs[key] = sig.keyword2[key].call(value)
        return f(*args, **kwargs)
    return w



from . import lib
from ..signature import Signature
from ..exceptions import HoretuCouldNotParse

@lib.Input
def django(function, argv):
    section = XXX
    kwargs = Signature(sec.func).keyword2

    return dict(section=section, func=sec.func,
                positionals=positionals, flags=flags)

    section_name = ''
    routes = {subcommand_dest: subcommand_tree}
    while isinstance(routes, dict):
        for k in list(routes):
            if hasattr(args, k):
                if getattr(args, k) is None:
                    p.print_usage()
                    sys.exit(2)
                g = routes[k][getattr(args, k)]
                routes = routes[k]
                section_name = util.extend(section_name, k)
                break
        else:
            break
    return g(p.parse_args(args))

import re
from functools import partial
import operator
from inspect import signature, Parameter
from configparser import ConfigParser
from collections import Counter

# from . import options

FLAG = re.compile(r'^-?(-[^-]).*')
# Step = options.Step

def _filename(x):
    if not os.path.isfile(x):
        raise ValueError('"%s" is not a file.' % x)
    return x


def one(parser, f):
    kind = None
    steps = []
    sfs = single_character_flags(sig)
    k2 = has_step(Step.keyword2, sig)
    for i, param in enumerate(sig.parameters.values()):
        st = step(kind, param)
        steps.append(st)

        args = options.choose_name_args(sfs, k2, st, param)
        argtype = options.argtype(param)
        config_file_arg_name = param.name

        if not((st == Step.keyword2 and k2) or \
            (st == Step.keyword1 and not k2)):
            if argtype == bool:
                raise TypeError('Cannot be bool: %s' % param)
            elif param.annotation == options.COUNT:
                raise TypeError('Cannot be COUNT: %s' % param)

        kwargs = dict(nargs=options.nargs(k2, st),
                      action=options.action(st, param),
                      type=argtype,
                      choices=options.argchoices(param),
                      help=helps.get(param.name, ''))

        if kwargs['action'] in {'store_true', 'store_false'}:
            pass
        elif config_file_arg_name in defaults:
            kwargs['default'] = argtype(defaults[config_file_arg_name])
        elif param.default != param.empty:
            kwargs['default'] = param.default
        if args[0].startswith('-'):
            kwargs['dest'] = param.name

        if kwargs['action'] in {'store_true', 'store_false', 'count'}:
            del(kwargs['choices'])
            del(kwargs['type'])
            del(kwargs['nargs'])
        parser.add_argument(*args, **kwargs)

    def g(parsed_args):
        args = []
        kwargs = {}

        kind = None
        for i, param in enumerate(sig.parameters.values()):
            st = step(kind, param)
            arg = getattr(parsed_args, param.name)
            if st == Step.positional:
                args.append(arg)
            elif st == Step.var_positional:
                args.extend(arg)
            else:
                kwargs[param.name] = arg
        return f(*args, **kwargs)
    return g

'''
KINDS = {
    Step.positional: {
        Parameter.POSITIONAL_ONLY,
        Parameter.POSITIONAL_OR_KEYWORD
    },
    Step.keyword1: {Parameter.POSITIONAL_OR_KEYWORD},
    Step.var_positional: {Parameter.VAR_POSITIONAL},
    Step.keyword2: {Parameter.KEYWORD_ONLY},
}
'''

def has_step(the_step, sig):
    kind = None
    for i, param in enumerate(sig.parameters.values()):
        st = step(kind, param)
        if st == the_step:
            return True
    return False

def single_character_flags(sig):
    kind = None
    x = ['-h']
    k2 = has_step(Step.keyword2, sig)
    for i, param in enumerate(sig.parameters.values()):
        st = step(kind, param)
        if k2:
            if st == Step.keyword2:
                x.append('-' + param.name[0])
        else:
            if st == Step.keyword1:
                x.append('-' + param.name[0])
    return set(name for name,count in Counter(x).items() if count > 1)

def step(prev_kind, param):
    if param.kind in KINDS[Step.positional].union(KINDS[Step.keyword1]):
        if param.default == param.empty:
            this_kind = Step.positional
        else:
            this_kind = Step.keyword1
    elif param.kind in KINDS[Step.var_positional]:
        this_kind = Step.var_positional
    elif param.kind in KINDS[Step.keyword2]:
        this_kind = Step.keyword2
    else:
        raise ValueError(
            'Variable keyword args (**kwargs) are not allowed. You may implement your own key-value parser that takes the result of variable positional args (*args).')

    if prev_kind and this_kind < prev_kind:
        raise ValueError('This should not happen.')

    return this_kind

def sub(config_file, config_section, subparsers, fs):
    g = {}
    for f in fs:
        name = f.__name__
        sp = subparsers.add_parser(name)
        g[name] = one(config_file, extend(config_section, name), sp, f)
    return g


def nest(config_file, config_section,
         subparsers, commands=[], subcommands={}):
    if not isinstance(commands, list):
        raise TypeError('commands must be a list.')
    if not isinstance(subcommands, dict):
        raise TypeError('subcommands must be a dict.')

    output = sub(config_file, config_section, subparsers, commands)

    for dest, subcommand in subcommands.items():
        subparser = subparsers.add_parser(dest)
        subsection = extend(config_section, dest)
        if isinstance(subcommand, dict):
            subsubparsers = subparser.add_subparsers(dest=dest)
            output[dest] = nest(config_file, subsection, subsubparsers,
                                subcommands=subcommand)
        elif hasattr(subcommand, '__call__'):
            output[dest] = one(config_file, subsection, subparser, subcommand)
        else:
            raise TypeError

    return output


def expand_dict_keys(x):
    for k, v in x.items():
        if hasattr(v, 'items'):
            for subk in expand_dict_keys(v):
                yield k + ' ' + subk
        else:
            yield k


def extend(a, b):
    return ((a or '') + ' ' + b).lstrip()



from copy import copy
from functools import partial

def funmap(*keys, name=None):
    _repr = 'funmap(' + ', '.join(map(repr, keys)) + ', name=%s)'
    class Repr(type):
        def __repr__(Class):
            return _repr % repr(Class.__name__)
    class Map(object):
        'This is strange so that :py:func:`isinstance` can work.'
        def __repr__(self):
            return _repr % repr(self.__name__)

        @classmethod
        def from_dict(FM, x):
            return FM(lambda:x)()

        def __new__(Class, func):
            class FunMap(Class, metaclass=Repr):
                __doc__ = func.__doc__
                def __new__(C, *args, **kwargs):
                    return object.__new__(C)
                
                def __repr__(self):
                    out = self.__class__.__name__ + '('
                    out += ', '.join(map(repr, self._args))
                    if self._kwargs:
                        out += ', '.join('%s=%s' % (k, repr(v)) \
                                         for k, v in self._kwargs.items())
                    out += ')'
                    return out

                def __str__(self):
                    tpl = '    %s: %s'
                    title = [repr(self)]
                    data = [(tpl % (k, self.__dict__.get(k,''))) \
                            for k in keys]
                    return '\n'.join(title + data)

                @classmethod
                def from_dict(FM, x):
                    fm = FM.__new__(FM)
                    fm._args = fm._kwargs = tuple()
                    fm._update(x)
                    return fm

                def __init__(self, *args, **kwargs):
                    self._args = tuple(args)
                    self._kwargs = dict(kwargs)
                    out = func(*self._args, **self._kwargs)

                    if not hasattr(out, 'items'):
                        raise ValueError('Function should return a dict.')
                    elif not hasattr(out, '__iter__') or set(out) != set(keys):
                        strk = '", "'.join(keys)
                        raise ValueError('Function should return a dict with "%s".' % strk)
                    self._update(out)

                def _update(self, x):
                    self._evaluated = True
                    for k, v in x.items():
                        if hasattr(self, k):
                            raise ValueError('Key "%s" is not allowed.' % k)
                        self.__dict__[k] = v
                
            FunMap.__name__ = func.__name__
            return FunMap
    if name:
        Map.__name__ = name
    return Map



from ...signature import COUNT

def nargs(has_k2, step):
    if has_k2 and step == Step.keyword1:
        return '?'
    elif step == Step.var_positional:
        return '*'

def dest(param):
    if param.annotation in {list, COUNT} and param.default != param.empty:
        return singularize(param.name)

def action(step, param):
    if param.call == COUNT:
        return 'count'

    if step == Step.positional:
        return 'store'
    elif step in {Step.keyword1, Step.keyword2}:
#       print(param.call, 88)
        if param.call == None:
            if param.default == True:
                return 'store_false'
            elif param.default == False:
                return 'store_true'
            else:
                return 'store'
        elif param.call == bool:
            if param.default == True:
                return 'store_false'
            else:
                return 'store_true'
        elif param.call == list:
            return 'append'
        else:
            return 'store'
    elif step == Step.var_positional:
        return 'store'

from enum import Enum
class Step(Enum):
    positional = 1
    keyword1 = 2
    var_positional = 3
    keyword2 = 4

def choose_name_args(single_character_flags, has_k2, st, param):
    if st == Step.positional or (has_k2 and st == Step.keyword1):
        args = param.name,
    elif st in {Step.keyword1, Step.keyword2}:
        lf = longflag(param)
        sf = shortflag(param)
        if lf and sf in single_character_flags:
            args = lf,
        else:
            if lf:
                args = sf, lf
            else:
                args = sf,
    elif st == Step.var_positional:
        args = param.name,
    else:
        raise ValueError('Bad step: %s' % st)
    return args




import os
import sys
import logging
from functools import partial, wraps
from collections import namedtuple

# Begin exports.
from . import input, output, render, exceptions
from . import templates
from .util import config_template, CallableDict
from .signature import COUNT
from .exceptions import Error

logger = logging.getLogger(__name__)

def Interface(function, config_file, config_sep, i):
    '''
    :param i: Function that takes no arguments and returns an input.Input
    '''
    try:
        body = output.Output(config_file, config_sep, function, i())
    except exceptions.HoretuException as e:
        body = e
    return body

WebResponse = namedtuple('WebResponse', ['content_type', 'data'])
def content_type(x):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return WebResponse(x, func(*args, **kwargs))
        return wrapper
    return decorator

def wsgi(function, url=None,
         name=None, config_file=None, config_sep=None,
         default_content_type='text/plain; charset=UTF-8', debug=False):
    '''
    :type f: Callable, list, or dict
    :param f: The callable to produce the argument parser too,
        or a dict of (dicts of...) str to callable to make subparsers.
    :type config_file: str or None
    :param config_file: Configuration file location, or None to disable
    :param str config_sep: Separator character between command and
        subcommand names in the configuration file (Default is " ".)
    :param str name: Name of the program, used for environment variable settings
        If it is ``None`` (the default), we attempt to get the name from the function.
    :param str url: If this is set, simply evaluate the function for that particular URL,
        rather than creating a WSGI application. This is nice for testing.
    :param str default_content_type: Content-Type with which pages should be served
    :param bool debug: Print errors and debugging information to web pages?
    :rtype: function
    :returns: WSGI application
    '''
    import traceback
    try:
        from webob import Response
    except ImportError:
        logger.error('WebOb is not installed; run this: pip install horetu[all]')
        sys.exit(1)

    def app(environ, start_response):
        res = Response()
        i = partial(input.wsgi, function, environ)
        try:
            out = Interface(config_file, config_sep, function, i)
        except Exception:
            if debug:
                res.content_type = 'text/plain; charset=UTF-8'
                body = [traceback.format_exception().encode('utf-8')]
            else:
                raise

        if isinstance(out, exceptions.HoretuCouldNotParse):
            res.status = 404
            wr = WebResponse('text/html',
                    templates.AppIter(render.wsgi.usage(function, out)))
        elif isinstance(out, exceptions.HoretuShowHelp):
            res.status = 200
            wr = WebResponse('text/html',
                    templates.AppIter(render.wsgi.man(function, out)))
        elif isinstance(out, exceptions.HoretuError):
            res.status = 400
            if isinstance(out, WebResponse):
                wr = out
            else:
                wr = WebResponse(default_content_type, out)
        else:
            res.status = 200
            if isinstance(out, WebResponse):
                wr = out
            else:
                wr = WebResponse(default_content_type, out)

        if wr.content_type.lower().startswith('text/plain'):
            res.app_iter = (chunk + b'\n' for chunk in wr.data)
        else:
            res.app_iter = wr.data
        res.content_type = wr.content_type
        return res(environ, start_response)
    return app

def cli(function, argv=sys.argv, return_=False,
        name=None, config_file=None, config_sep=None,
        exit_=sys.exit, stdout=sys.stdout.buffer, stderr=sys.stderr.buffer):
    '''
    :type f: Callable, list, or dict
    :param f: The callable to produce the argument parser too,
        or a dict of (dicts of...) str to callable to make subparsers.
    :type config_file: str or None
    :param config_file: Configuration file location, or None to disable
    :param str config_sep: Separator character between command and
        subcommand names in the configuration file (Default is " ".)
    :param str name: Name of the program (``$0``), for usage text and
        environment variables. If it is ``None`` (the default), we
        use the following instead.
       
        1. If a function, rather than a list or dictionary, is passed as ``f``,
           use the function's name (``f.__name__``).
        2. Otherwise, use the first argv (``sys.argv[0]``).
    :param list argv: :py:obj:`sys.argv` by default
    :param bool return_: If this is ``True``, simply return the result of the function;
        do not write stuff to ``exit_``, ``stdout``, or ``stderr``.
    :param function exit_: :py:func:`sys.exit` by default
    :type stdout: Binary file
    :param stdout: :py:func:`sys.stdout.buffer` by default
    :type stderr: Binary file
    :param stderr: :py:func:`sys.stderr.buffer` by default
    '''
    argv = list(argv) # Prevent myself from modifying the input.

    if name is None:
        if hasattr(function, '__call__'):
            name = function.__name__
        else:
            name = os.path.basename(argv[0])

    try:
        out = Interface(function, config_file, config_sep,           # pass through
                        partial(input.cli, function, argv[1:])) # input
    except exceptions.Error as e:
        out = e

    code = 0
    fp = stdout
    if isinstance(out, exceptions.HoretuException):
        if not out.ok:
            fp = stderr

        if isinstance(out, exceptions.HoretuShowHelp):
            app_iter = templates.AppIter(render.cli.man(name, function, out))
        else:
            if isinstance(out, exceptions.HoretuCouldNotParse):
                code = 2
            else:
                code = 1
            app_iter = templates.AppIter(render.cli.usage(name, function, out))
    else:
        app_iter = out
    _communicate_cli(stdout, stderr, fp, app_iter)
    exit_(code)

def _communicate_cli(stdout, stderr, fp, app_iter):
    try:
        for line in app_iter:
            fp.write(line + b'\n')
    except BrokenPipeError:
        stderr.close()
    except exceptions.Error as e:
        stdout.buffer.write(e.message.rstrip('\n').encode('utf-8') + b'\n')

def django(function):
    try:
        from django.core.management.base import BaseCommand, CommandError
    except ImportError:
        logger.error('Django is not installed; run this: pip install horetu[all]')
        sys.exit(1)
    class Command(BaseCommand):
        def add_arguments(self, parser):
            render.argparse.update_parser(function, parser)
        def handle(self, *args, **options):
            fp = self.stdout
            try:
                y = render.argparse.evaluate(function, args, options)
            except Exception as e:
                raise CommandError(str(e))
            else:
                _communicate_cli(self.stdout, self.stderr, fp.buffer,
                                 templates.AppIter(y))
    return Command

@wraps(cli)
def horetu(*args, **kwargs):
    logger.warn('The horetu.horetu interface is deprecated; use horetu.cli.')

    if 'description' in kwargs:
        del(kwargs['description'])

    if 'args' in kwargs:
        kwargs['argv'] = sys.argv[:1] + kwargs.pop('args')

    return cli(*args, **kwargs)

def irc(function, config=None, config_sep=' ',
        ):
    '''
    Render Python functions as an IRC bot.

    :type f: Callable, list, or dict
    :param f: The callable to produce the argument parser too,
        or a dict of (dicts of...) str to callable to make subparsers.
    :type config: str or None
    :param config: Configuration file location, or None to disable
    :param str config_sep: Separator character between command and
        subcommand names in the configuration file (Default is " ".)
    '''
    raise NotImplementedError

def webcli(function, config=None, config_sep=' '):
    '''
    Render Python functions as text terminal website.

    :type f: Callable, list, or dict
    :param f: The callable to produce the argument parser too,
        or a dict of (dicts of...) str to callable to make subparsers.
    :type config: str or None
    :param config: Configuration file location, or None to disable
    :param str config_sep: Separator character between command and
        subcommand names in the configuration file (Default is " ".)
    '''
    raise NotImplementedError



class HoretuException(Exception):
    def __init__(self, message, section=None):
        # message first so it is easy not to provide a section
        if section:
            self.section = section
        else:
            self.section = tuple()
        self.message = message

    def __repr__(self):
        return '%s(%s, section=%s)' % (
            self.__class__.__name__,
            repr(self.message),
            repr(self.section),
        )
    __str__ = __repr__

class HoretuError(HoretuException):
    ok = False

class HoretuCouldNotParse(HoretuError):
    pass

class Error(HoretuError):
    pass

class HoretuShowHelp(HoretuException):
    ok = True



import logging
import itertools
import inspect
from functools import wraps
from funmap import funmap
from triedict import triedict
from collections import namedtuple
from copy import copy
from .exceptions import HoretuShowHelp, HoretuCouldNotParse
from enum import Enum
from .util import CallableDict
from .lib import validate_param_name

logger = logging.getLogger(__name__)

class COUNT(object):
    def __init__(self):
        self.count = 0
    def __call__(self, _):
        self.count += 1
        return self.count

class LIST(object):
    def __init__(self):
        self.xs = 0
    def __call__(self, x):
        self.xs.append(x)
        return self.xs

def show_help(x):
    raise HoretuShowHelp(x)

@funmap('name', 'call', 'takes_parameter', 'supports_positional', 'default')
def Annotation(param):
    '''
    Determine the type of a parameter. For parameters that take
    arguments.

    :param inspect.Parameter param: Parameter from the function signature
    :ivar str name: Argument name
    :ivar call: Parser function that takes one str argument as input
        (Ignore the input if you don't need it.)
    :ivar bool takes_parameter: Whether the keyword argument takes a parameter
    :ivar bool supports_positional: Whether it can be used as a positional
    :ivar default: Default value, for keyword arguments (used for
        generating default configuration files)
    '''
    if param.annotation == param.empty:
        if isinstance(param.default, bool) and \
                param.default in {True, False}:
            y = lambda _: not param.default, False, False
        else:
            y = str, True, True
    elif param.annotation == bool:
        y = lambda _: not param.default, False, False
    elif isinstance(param.annotation, tuple):
        y = Factor(copy(param.annotation)), True, True
    elif param.annotation == COUNT:
        y = COUNT(), False, False
    elif param.annotation == list:
        y = LIST(), True, False
    elif hasattr(param.annotation, '__call__'):
        y = param.annotation, True, True
    else:
        raise ValueError('Bad input annotation: %s' % param.annotation)

    a_, b, c = y
    def a(x):
        try:
            return a_(x)
        except HoretuCouldNotParse:
            raise
        except Exception as e:
            raise HoretuCouldNotParse('Could not parse %s: %s' %
                                      (param.name, str(e)))

    return {'name': param.name, 'default': param.default,
            'call':a, 'takes_parameter':b, 'supports_positional':c}

help_annotation = Annotation.from_dict({
    'name': 'help',
    'call': show_help,
    'takes_parameter': False,
    'supports_positional': False,
    'default': None,
})

def Factor(options):
    if all(isinstance(option, str) for option in options):
        def enum(x):
            if x in options:
                return x
            else:
                raise HoretuCouldNotParse('Must be one of %s' % str(options))
        return enum
    else:
        raise ValueError('All options must be of str type.')

@funmap('positional', 'keyword1', 'var_positional', 'keyword2')
def Signature(f, with_help=True):
    '''
    Restructure the argument information.

    :ivar list positional:
    :ivar list keyword1:
    :ivar Arg var_positional:
    :ivar dict keyword2:
    '''
    x = {
        'positional': [],
        'keyword1': [],
        'var_positional': None,
        'keyword2': triedict({'help': help_annotation} if with_help else {}),
    }
    
    sig = inspect.signature(f.func if isinstance(f, CallableDict) else f)

    for param in sig.parameters.values():
        validate_param_name(param.name)
        if param.kind == inspect.Parameter.VAR_KEYWORD and \
                not isinstance(f, CallableDict):
            raise ValueError(VAR_KWARGS)

    for kind, param in kinds(sig):
        a = Annotation(param)
        if kind == Kind.positional:
            x['positional'].append(a)
        elif kind == Kind.keyword1:
            x['keyword1'].append(a)
        elif kind == Kind.var_positional:
            x['var_positional'] = a
        elif kind == Kind.keyword2:
            x['keyword2'][param.name] = a
        else:
            raise TypeError(kind)

    return x

class Kind(Enum):
    positional = 1
    keyword1 = 2
    var_positional = 3
    keyword2 = 4

    # https://docs.python.org/3/library/enum.html#orderedenum
    def __ge__(self, other):
        if self.__class__ is other.__class__:
            return self.value >= other.value
        return NotImplemented
    def __gt__(self, other):
        if self.__class__ is other.__class__:
            return self.value > other.value
        return NotImplemented
    def __le__(self, other):
        if self.__class__ is other.__class__:
            return self.value <= other.value
        return NotImplemented
    def __lt__(self, other):
        if self.__class__ is other.__class__:
            return self.value < other.value
        return NotImplemented

KINDS = {
    Kind.positional: {
        inspect.Parameter.POSITIONAL_ONLY,
        inspect.Parameter.POSITIONAL_OR_KEYWORD
    },
    Kind.keyword1: {inspect.Parameter.POSITIONAL_OR_KEYWORD},
    Kind.var_positional: {inspect.Parameter.VAR_POSITIONAL},
    Kind.keyword2: {inspect.Parameter.KEYWORD_ONLY},
}


VAR_KWARGS = 'Variable keyword args (**kwargs) are not allowed. You may implement your own key-value parser that takes the result of variable positional args (*args).'

def kinds(sig):
    has_k2 = Kind.keyword2 in (k for k,_ in _naive_kinds(sig))
    for kind, param in _naive_kinds(sig):
        if kind == Kind.keyword1 and not has_k2:
            yield Kind.keyword2, param
        else:
            yield kind, param

def _naive_kinds(sig):
    for param in sig.parameters.values():
        if param.kind in KINDS[Kind.positional].union(KINDS[Kind.keyword1]):
            if param.default == param.empty:
                kind = Kind.positional
            else:
                kind = Kind.keyword1
        elif param.kind in KINDS[Kind.var_positional]:
            kind = Kind.var_positional
        elif param.kind in KINDS[Kind.keyword2]:
            kind = Kind.keyword2
        yield kind, param

def TOC(prog, root_section):
    for section in sorted(prog):
        if section[:len(root_section)] == root_section:
            yield section, Signature(prog[section])

def serialize_endpoint(func):
    def endpoints(prog, section):
        for args in TOC(prog, section):
            yield func(*args)
    return endpoints



import re
from functools import partial
import operator
from collections import Counter, ChainMap

from . import options
from ... import exceptions
from ..doc import Doc
from ...signature import Signature

FLAG = re.compile(r'^-?(-[^-]).*')
Step = options.Step

def one(parser, f):
    s = Signature(f, with_help=False)
    d = Doc(f)
    kw = partial(get_kwargs,
                 ChainMap(d.args, d.kwargs),
                 len(s.keyword2) > 0)

    for param in s.positional:
        parser.add_argument(param.name, **kw(Step.positional, param))
    if s.var_positional:
        parser.add_argument(s.var_positional.name,
                            **kw(Step.positional, s.var_positional))
    for param in s.keyword1:
        parser.add_argument(s.keyword1.name, **kw(Step.keyword1, param))
    for name, param in s.keyword2.items():
        parser.add_argument('--' + name, **kw(Step.keyword2, param))

def get_kwargs(helps, k2, step, param):
    def call(x):
        try:
            return param.call(x)
        except exceptions.HoretuCouldNotParse as e:
            raise ValueError(e.message)
    kwargs = dict(nargs=options.nargs(k2, step),
                  action=options.action(step, param),
                  type=call,
                  default=param.default,
                  help=helps.get(param.name))
    if kwargs['action'] in {'store_true', 'store_false', 'count'}:
        del(kwargs['type'])
        del(kwargs['nargs'])
    return kwargs



import argparse
import sys
import os
from functools import partial
from itertools import chain

from ...signature import Signature
from ..doc import Doc
from .one import one
from . import options

def update_parser(f, p):
    '''
    :param f: Function
    :param argparse.ArgumentParser p: Parser
    '''
    subcommand_dest = 'subcommands'
    defaults = {}

    p.formatter_class=argparse.RawDescriptionHelpFormatter
    if hasattr(f, '__call__'):
        if p.description is None:
            p.description = Doc(f).desc
        one(p, f)
    elif isinstance(f, (dict, list)):
        raise ValueError('Subcommands are not allowed in the argparse/django interface.')
    else:
        raise TypeError('f must be callable.')

def evaluate(f, args, options):
    args = list(args)
    kwargs = {}

    s = Signature(f, with_help=False)
    for annotation in s.positional:
        args.append(options[annotation.name])
    if s.var_positional:
        args.extend(options[s.var_positional.name])
    for annotation in s.keyword1:
        kwargs[annotation.name] = options[annotation.name]
    for annotation in s.keyword2.values():
        kwargs[annotation.name] = options[annotation.name]

    return f(*args, **kwargs)



__all__ = ['triedict']
from collections import defaultdict, MutableMapping

class AmbiguousKey(KeyError):
    pass

class KeyNotFound(KeyError):
    pass

def _dictdefaultdict():
    return defaultdict(_dictdefaultdict)

def _chars(key):
    for i in range(len(key)):
        yield key[i:i+1]

def _iter_trie_dict(prefix, d):
    for k, v in d.items():
        if k == None:
            yield prefix
        else:
            yield from _iter_trie_dict(prefix + k, v)

class triedict(MutableMapping):
    def __init__(self, *args, **kwargs):
        self._type = None
        self._trie = _dictdefaultdict()
        self.update(dict(*args, **kwargs))

    def _check_type(self, key):
        if self._type:
            if isinstance(key, self._type):
                pass # good
            else:
                raise TypeError('Key must be of %s type.' % self._type)
        else:
            if hasattr(key, '__iter__'):
                self._type = type(key)
            else:
                raise TypeError('Key must be iterable.')

    def __repr__(self):
        return 'triedict(%s)' % repr(dict(self.items()))

    def __getitem__(self, key):
        self._check_type(key)
        d = self._trie
        subkey = [None] + list(key)
        for char in _chars(key):
            if char in d:
                d = d[char]
                subkey = subkey[1:]
            else:
                break
        if d == self._trie and len(key) > 0:
            raise KeyNotFound('Not found: ' + str(key))

        if None in d:
            if len(subkey) == 1:
                return d[None]
            else:
                raise KeyNotFound('Not found: ' + str(key))
        else:
            while None not in d and len(d) == 1:
                d = d[next(iter(d))]
                subkey = subkey[1:]

            if len(d) > 1:
                if (not subkey) or (subkey[0] in d):
                    raise AmbiguousKey('Ambiguous: ' + str(key))
                else:
                    raise KeyNotFound('Not found: ' + str(key))
            elif len(d) == 0 or None not in d:
                raise KeyNotFound('Not found: ' + str(key))
            else:
                return d[None]

    def __setitem__(self, key, value):
        self._check_type(key)
        d = self._trie
        for char in _chars(key):
            d = d[char]
        d[None] = value

    def __delitem__(self, key):
        self._check_type(key)
        raise NotImplementedError

    def __iter__(self):
        d = self._trie
        if d:
            return _iter_trie_dict(self._type(), d)
        else:
            return iter([])

    def __len__(self):
        n = 0
        for _ in self:
            n += 1
        return n



import logging
from funmap import funmap
from triedict import triedict

logger = logging.getLogger(__name__)

def validate_param_name(*xs):
    msg = 'It is best if you remove "%s" from the argument name "%s".'
    for x in xs:
        for c in '/?[]<>():':
            if x in c:
                logger.warning(msg % (c, x))
    
def Program(functions):
    '''
    :param functions: Base function or functions
    :rtype: dict
    :returns: Map path tuples to functions.
    '''
    return triedict(_sub(tuple(), functions))

def _sub(section, fs):
    '''
    :param tuple section: Sections that this sub-program is under
    '''
    if hasattr(fs, '__call__'):
        yield section, fs, 

    if hasattr(fs, 'items'):
        items = fs.items()
    elif hasattr(fs, '__iter__'):
        items = ((f.__name__,f) for f in fs)
    else:
        items = []
    for name, f in items:
        validate_param_name(*name)
        yield from _sub(section + (name,), f)



from . import lib
from ..signature import Signature
from ..exceptions import HoretuCouldNotParse

@lib.Input
def cli(function, argv):
    sec = lib.getsection(True, function, argv)
    kwargs = Signature(sec.func).keyword2

    mixed = iter(argv[len(sec.section):])
    positionals = []
    flags = []
    before_double_hyphen = True
    while True:
        try:
            raw_arg = next(mixed)
        except StopIteration:
            break

        arg = raw_arg[1:]
        if before_double_hyphen and _yes_hyphen(raw_arg):
            if raw_arg == '--':
                before_double_hyphen = False
            elif arg in kwargs:
                x = kwargs[arg]
                if x.takes_parameter:
                    try:
                        value = next(mixed)
                    except StopIteration:
                        raise HoretuCouldNotParse('Needs parameter -- %s' % arg)
                else:
                    value = None
                flags.append((x.name, value))
            else:
                raise HoretuCouldNotParse('Unknown option -- %s' % arg)
        else:
            positionals.append(raw_arg)

    return dict(section=sec.section, func=sec.func,
                positionals=positionals, flags=flags)

def _yes_hyphen(x):
    return x.startswith('-')

def _no_hyphen(x):
    return not x.startswith('-')



import re
from itertools import filterfalse
from funmap import funmap
from inspect import getdoc
from ..signature import Signature

HELP = ('help', 'Display this help.')

@funmap('desc', 'args', 'kwargs')
def Doc(f):
    if not f:
        return {'desc': '', 'args': [], 'kwargs': [HELP]}
    lines = _doclines(f)
    colons = _read_colons(lines)
    s = Signature(f)

    _argnames = set(p.name for p in s.positional + s.keyword1)
    if s.var_positional:
        _argnames.add(s.var_positional.name)
    _kwargnames = set(p.name for p in s.keyword2.values())

    _args = []
    _kwargs = []

    for pair in colons:
        name, desc = pair
        if name in _argnames:
            _args.append(pair)
        else:
            _kwargs.append(pair)

    for name in _argnames:
        if name not in dict(_args):
            _args.append((name, 'Undocumented'))

    if not 'help' in dict(_kwargs):
        _kwargs.append(HELP)

    for name in _kwargnames:
        if name not in dict(_kwargs):
            _kwargs.append((name, 'Undocumented'))

    return {
        'desc': _desc(lines),
        'args': _args,
        'kwargs': _kwargs,
    }

COLON = re.compile(r'^[\s:]')
def _colon(line):
    return bool(re.match(COLON, line))

def _desc(lines):
    return '\n'.join(filterfalse(_colon, lines)).strip()

PARAM = re.compile(r'^:param (?:[^:]+ )?([^:]+): (.+)$')
INDENT = re.compile(r'\s+([^\s].*)')

def _read_colons(lines):
    y = []

    name = None
    for line in lines:
        if _colon(line):
            m = re.match(PARAM, line)
            n = re.match(INDENT, line)
            if m:
                # New parameter
                if name:
                    # Save the previous parameter if it exists.
                    y.append((name, desc.strip()))
                    name = None

                # Start the new description
                name, desc = m.groups()
            elif name and n:
                desc += ' ' + n.group(1)
    if name:
        # Save the last parameter
        y.append((name, desc.strip()))
    
    return y

def _doclines(f):
    d = getdoc(f)
    if not d:
        return ''
    return d.split('\n')



import logging
from io import StringIO
from pprint import pformat
from collections import defaultdict
from configparser import ConfigParser
from .signature import Signature

logger = logging.getLogger(__name__)

MAIN = 'dummy main section'

def write(prog, config_fp, config_sep):
    '''
    :param Program prog: Function, nested functions, &c.
    :type config_fp: File-like object with text write flags
    :param config_fp: Configuration file
    :type config_sep: str or NoneType
    :param config_sep: Character to separate sections, or None to indicate no sectioning
    '''
    if config_sep:
        d = _rdict()
        for section in prog:
            str_section = config_sep.join(section)
            d[str_section] = {}
            for key, param in Signature(prog[section]).keyword2.items():
                d[str_section][key] = param
    else:
        d = {}
        for section in prog:
            for key, param in Signature(prog[section]).keyword2.items():
                if key in d:
                    if param.default != d[key]:
                        msg = 'There are different defaults for -%s. Make the defaults match, use different names, or set a config_sep.'

                        raise ValueError(msg % key)
                else:
                    d[key] = param
        d = {MAIN: d}
    
    c = ConfigParser()
    for section, contents in sorted(d.items()):
        c.add_section(section)
        for k in sorted(contents):
            param = contents[k]
            try:
                if param.call(repr(param.default)) == param.default:
                    c[section][k] = repr(param.default)
            except Exception:
                pass

    with StringIO() as gp:
        c.write(gp)
        gp.seek(0)
        if not config_sep:
            gp.readline()
        config_fp.write(gp.read())

def read(config_fp, config_sep):
    '''
    :type config_fp: File-like object with text read flags
    :param config_fp: Configuration file
    :type config_sep: str or NoneType
    :param config_sep: Character to separate sections, or None to indicate no sectioning
    :rtype: dict
    :returns: Stuff from the config file
    '''
    d = _rdict()

    if config_fp:
        if config_sep:
            fp = config_fp
        else:
            fp = StringIO()
            fp.write('[%s]\n' % MAIN)
            fp.write(config_fp.read())
            fp.seek(0)

        c = ConfigParser()
        c.read_file(fp)

        if config_sep:
            for config_section in c.sections():
                y = d
                if config_sep:
                    for component in config_section.split(config_sep):
                        y = y[component]
                y.update(c[config_section])

                w = (config_section, pformat(y))
                logger.info('Defaults from section [%s]:\n%s' % w)

            d = _as_dict(d)
        else:
            d = c[MAIN]

    else:
        d = {}
    return d

def _rdict():
    return defaultdict(_rdict)

def _as_dict(x):
    if isinstance(x, defaultdict):
        return {k: _as_dict(v) for k, v in x.items()}
    else:
        return x



from distutils.core import setup

packages=[
    'horetu',
    'funmap',
    'triedict',
    'horetu.input', 
    'horetu.render',
    'horetu.render.argparse',
]

setup(name='horetu',
      author='Thomas Levine',
      author_email='_@thomaslevine.com',
      description='Make a command-line interface from a function.',
      url='http://src.thomaslevine.com/horetu/',
      packages=packages,
      install_requires=[
      ],
      extras_require={
          'all': ['horetu[wsgi]', 'horetu[django]'],
          'django': ['Django>=1.10.2'],
          'docs': ['sphinxcontrib-autorun>=0.1'],
          'wsgi': ['WebOb>=1.6.1'],
          'tests': ['pytest>=2.6.4'],
          'dev': ['horetu[docs]', 'horetu[tests]', 'horetu[all]'],
      },
      tests_require=[
          'horetu[tests]',
      ],
      classifiers=[
          'Programming Language :: Python :: 3.5',
      ],
      version='0.3.1',
      license='AGPL',
      )



from functools import update_wrapper

class CallableDict(dict):
    '''
    Merge function with dictionary. This is useful for specifying a root
    command for horetu. ::

        f = CallableDict(lambda x:'You sent %s to the root command.' % x,
                         subcommand=lambda a,b:a+b)
        horetu.cli(f)

    You could also use it as a decorator ::

        @CallableDict
        def f(x):
            return x+3
        f['four'] = lambda: x+4

    And you can nest them. ::

        f['sub'] = CallableDict(lambda a,b,c:a*b-c, hi=lambda:8)

    :param func: The function to use as the main command
    :param args: Passed to :py:class:`dict`
    :param kwargs: Passed to :py:class:`dict`
    '''
    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.__name__ = func.__name__
        super(CallableDict, self).__init__(*args, **kwargs)
        update_wrapper(self, func)

    def __call__(self, *args, **kwargs):
        return self.func(*args, **kwargs) 
    def __copy__(self):
        return CallableDict.new(self.func, self)

def config_template(function):
    '''
    Write a configuration file with all the default parameter values.
    You can import this in your program to add a flag for creating a
    template configuration file. ::

        def main(x, y, z, sample_config=False):
            if sample_config:
                sys.stdout.write(horetu.config_template(main))
            else:
                ...

    Or, rather than including it in your end-user program, just call it
    once to create a template, and distribute that.
    '''
    raise NotImplementedError
